module game {
    export class BulletData {
        scale: number;
        speedMin: number;
        speedMax: number;
        earthDamage: number;
        g: number;
        hard: number;
        boomRange: number;
        res: string;
        resAnimation: string;

        boomSpine: string;
        boomAnimation: string;
        boomScale: number;
        shoujiSpine: string;
        shoujiAnimation: string;
        shoujiScale: number;
        damageType: string;
        filpX: number;

        m_speed: number;
        m_dir: cc.Vec2;
        m_pos: cc.Vec2;
        m_angle: number;
        m_roledir: number;
        m_roleId: number;
        m_isplayer: boolean;

        toString() {
            return "scale :" + this.scale + "\n"
                + "speedMin :" + this.speedMin + "\n"
                + "speedMax :" + this.speedMax + "\n"
                + "earthDamage :" + this.earthDamage + "\n"
                + "g :" + this.g + "\n"
                + "hard :" + this.hard + "\n"
                + "boomRange :" + this.boomRange + "\n"
                + "boomSpine :" + this.boomSpine + "\n"
                + "boomScale :" + this.boomScale + "\n"
                + "shoujiSpine :" + this.shoujiSpine + "\n"
                + "shoujiAnimation :" + this.shoujiAnimation + "\n"
                + "shoujiScale :" + this.shoujiScale + "\n"
                + "damageType :" + this.damageType + "\n"
                + "filpX :" + this.filpX + "\n";
        }
    }

    export class baseBullet extends xjNode {
        m_SpineNode: SkeletonAnimation_Ex;
        m_timer: number = 0;
        m_pos: cc.Vec2;
        // m_oldPos;
        m_roledir: number;
        m_bulletData: BulletData;
        m_canMove: boolean;
        m_scale:number;

        static create(data: BulletData) {
            let bullet = new baseBullet();
            bullet.initData(data);
            return bullet;
        }

        initData(data) {
            this.m_bulletData = data;
            this.loadres();
            this.setPosition(this.m_bulletData.m_pos);
            this.m_canMove = true;
            // this.m_oldPos = this.getPosition();
            this.m_roledir = this.m_bulletData.m_roledir;

            cameraMgr.getMe().reset(this.getPosition(), this);
        }

        canRotation() {
            return this.m_bulletData.filpX == 0;
            // return true;
        }

        updateRotation(oldPos: cc.Vec2, newPos: cc.Vec2) {
            let shootVector = cc.pNormalize(cc.pSub(oldPos, newPos));
            let degrees = cc.radiansToDegrees(cc.pToAngle(shootVector));
            if (this.m_bulletData.m_roledir > 0) {
                this.setRotation(180 - degrees);
            } else {
                this.setRotation(-degrees);
            }
        }

        update(dt) {
            if (!this.m_canMove) return;
            this.m_timer += dt;
            this.m_pos = this.getPosition();
            if (this.m_pos.x < 0 || this.m_pos.x > 2000 || this.m_pos.y < 0 || this.m_pos.y > 3000) {
                this.moveEnd();
                return;
            }
            this.m_pos.y -= this.m_timer * this.m_bulletData.g;
            this.m_pos = cc.pAdd(this.m_pos, cc.pMult(this.m_bulletData.m_dir, this.m_bulletData.m_speed * dt));
            // let x = this.m_bulletData.m_dir.x * this.m_bulletData.m_speed * dt;
            // let y = this.m_bulletData.m_dir.y * this.m_bulletData.m_speed * dt;
            // this.m_pos.x = this.m_pos.x +x;
            // this.m_pos.y = this.m_pos.x +y;

            // cc.log("time:",this.m_timer);
            /*            let x = this.m_bulletData.m_speed * this.m_bulletData.m_dir.x * this.m_timer;
             let y = ((this.m_bulletData.m_speed * this.m_bulletData.m_dir.y) * 2 - this.m_bulletData.g * this.m_timer) * this.m_timer / 2;
             this.m_pos = cc.pAdd(this.m_oldPos, cc.p(x, y));*/

            if (sceneMgr.getMe().getCurScene().m_tileMap.canMove(this.m_pos)) {
                if (this.canRotation()) {
                    this.updateRotation(this.getPosition(), this.m_pos);
                }
                this.move();
            } else {
                this.m_timer = 0;
                let hard = sceneMgr.getMe().getCurScene().m_tileMap.getGIDHard(this.m_pos);
                // cc.log(hard);
                this.m_bulletData.m_speed = this.m_bulletData.m_speed * this.m_bulletData.hard * hard;
                if (this.m_bulletData.m_speed < g_other_config.bulletMinSpeed) {
                    if (this.m_bulletData.damageType == "boom") {
                        this.boom(this.m_pos);
                    }
                    this.moveEnd();
                }
            }
        }

        getSpeed(){
            return this.m_bulletData.m_speed;
        }

        move() {
            this.setPosition(this.m_pos);
            this.dealphy(this.m_pos);
        }

        moveEnd() {
            this.m_canMove = false;
            cc.log("moveEnd");
            bulletMgr.getMe().removeBullet(this.__instanceId);
        }

        m_dealTarget: baseRole;

        dealphy(pos: cc.Vec2) {
            let roleMap;
            if (this.m_bulletData.m_isplayer) {
                if(sceneMgr.getMe().getCurScene() instanceof fightAiscene){
                    let rolemgr = <roleMgr_ai>roleMgr.getMe();
                    roleMap = rolemgr.m_matcher_map;
                }
                else if(sceneMgr.getMe().getCurScene() instanceof PVP_fight){
                    let rolemgr = <roleMgr_pvp>roleMgr.getMe();
                    roleMap = rolemgr.m_matcher_map;
                }
                else{
                    roleMap = roleMgr.getMe().m_monster_map;
                }
            }
            else {
                roleMap = roleMgr.getMe().m_player_map;
            }
            for (let i = 0; i < roleMap.size; ++i) {
                this.m_dealTarget = roleMap.getByIndex(i);
                if (this.m_dealTarget.isDead()) continue;
                if (this.m_dealTarget.dealPhy(pos)) {
                    if (this.m_bulletData.damageType == "boom") {
                        this.boom(pos);
                    } else {
                        this.hit();
                    }
                    this.moveEnd();
                    return;
                }
            }
        }

        hit() {
            cc.log("子弹撞击伤害", this.m_dealTarget.getName());
            this.m_dealTarget.onDamage(this.m_bulletData.shoujiSpine, this.m_bulletData.shoujiAnimation, this.m_bulletData.shoujiScale, this.m_bulletData.m_roleId, false);
        }

        boom(pos: cc.Vec2) {
            if (!this.m_dealTarget) {
                this.m_dealTarget = roleMgr.getMe().m_AllRole_map.getByIndex(this.m_bulletData.m_roleId);
            }
            cc.log("子弹会爆炸", this.m_dealTarget.getName());
            let ret = spineMgr.getMe().createSpineOncePlay(this.m_bulletData.boomSpine, this.m_bulletData.boomAnimation, roleMgr.getMe().getNode(), pos);
            ret.setScale(this.m_bulletData.boomScale);
            // 炸人 // 炸地面
            this.boomRole();
            this.boomGround(this.m_bulletData.boomRange);
        }

        boomRole() {
            for (let i = 0; i < roleMgr.getMe().m_AllRole_map.size; ++i) {
                this.m_dealTarget = roleMgr.getMe().m_AllRole_map.getByIndex(i);
                if (this.m_dealTarget.isDead()) continue;
                // if(cc.rectIntersectsRect(this.m_dealTarget.getshoujizudang(),this.getshoujizudang())){
                //     this.m_dealTarget.onDamage(this.m_bulletData.shoujiSpine, this.m_bulletData.shoujiAnimation, this.m_bulletData.shoujiScale, this.m_bulletData.m_roleId, true);
                // }
                // let dis = cc.pDistance(this.m_dealTarget.getBoneWroldPosition("shoujizudang"), this.getBoneWroldPosition("gongjizudang1"));
                // if (dis < this.m_bulletData.boomRange) {
                //     this.m_dealTarget.onDamage(this.m_bulletData.shoujiSpine, this.m_bulletData.shoujiAnimation, this.m_bulletData.shoujiScale, this.m_bulletData.m_roleId, true);
                // }

                this.dealBoomRole(this.m_dealTarget);
            }
        }

        dealBoomRole(role:baseRole){
            let rect = role.getshoujizudang();
            let p = cc.p(rect.x,rect.y);
            if(this.dealtwo(p,role)){
                return;
            }

            p = cc.p(rect.x,rect.y + rect.height);
            if(this.dealtwo(p,role)){
                return;
            }

            p = cc.p(rect.x + rect.width,rect.y + rect.height);
            if(this.dealtwo(p,role)){
                return;
            }

            p = cc.p(rect.x + rect.width,rect.y);
            if(this.dealtwo(p,role)){
                return;
            }

            p = cc.p(rect.x + rect.width / 2,rect.y + rect.height / 2);
            if(this.dealtwo(p,role)){
                return;
            }
        }

        dealtwo(p,role){
            let dis = cc.pDistance(p, this.getPosition());
            if (dis < this.m_bulletData.boomRange) {
                role.onDamage(this.m_bulletData.shoujiSpine, this.m_bulletData.shoujiAnimation, this.m_bulletData.shoujiScale, this.m_bulletData.m_roleId, true);
                return true;
            }
            return false;
        }

        boomGroundEx(range: number){
            if(range < 25){
                return 25;
            }
            if(range < 50){
                return 50;
            }
            if(range < 75){
                return 75;
            }
            if(range < 100){
                return 100;
            }
            if(range < 125){
                return 125;
            }
            if(range < 150){
                return 150;
            }
            if(range < 175){
                return 175;
            }
            if(range < 200){
                return 200;
            }
            if(range < 255){
                return 255;
            }
            return 0;
        }

        boomGround(range: number) {
            this.dealGroudByDegress(5, range);
            range = this.boomGroundEx(range);
            switch (range) {
                case 255:
                    this.dealGroudByDegress(5, 255);
                case 200:
                    this.dealGroudByDegress(5, 200);
                case 175:
                    this.dealGroudByDegress(5, 175);
                case 150:
                    this.dealGroudByDegress(10, 150);
                case 125:
                    this.dealGroudByDegress(10, 125);
                case 100:
                    this.dealGroudByDegress(20, 100);
                case 75:
                    this.dealGroudByDegress(20, 75);
                case 50:
                    this.dealGroudByDegress(20, 50);
                case 25:
                    this.dealGroudByDegress(20, 25);
                case 0:
                    this.dealGroudByDegress(360, 0);
            }

        }

        dealGroudByDegress(degress: number, distance: number) {
            for (let i = 0; i < 360 / degress; ++i) {
                sceneMgr.getMe().getCurScene().m_tileMap.reMoveSpriteByWorldPosition(pForDegressAndDistance(this.m_pos, 360 / degress * i, distance), this.m_bulletData.earthDamage, this.__instanceId);
            }
        }

        loadres() {
            this.m_SpineNode = spineMgr.getMe().loadSpine(this.m_bulletData.res, this.getNode(), 4);
            this.m_SpineNode.setAnimation(0, this.m_bulletData.resAnimation, true);
            this.m_SpineNode.setScale(this.m_bulletData.scale);
            this.m_scale = this.m_bulletData.scale;

            if (this.m_bulletData.m_roledir > 0) {
                this.m_SpineNode.setScaleX(-1 * this.m_SpineNode.getScale());
                this.setRotation(-this.m_bulletData.m_angle);
            } else {
                this.setRotation(180 - this.m_bulletData.m_angle);
            }
            return true;
        }

        zanting(flag) {
            if (flag) {
                this.m_SpineNode.pause();
            }
            else {
                this.m_SpineNode.resume();
            }
        }

        shutdown() {
            this.m_canMove = false;
            if (this.m_SpineNode) {
                this.removeChild(this.m_SpineNode.getNode());
                this.m_SpineNode = null;
            }
        }

        getshoujizudang(){
            let rect = this.m_SpineNode.getBoundingBoxBySlotName("gongjizudang1");
            let root = this.getPosition();
            rect.x = root.x + (rect.x - root.x) * this.m_scale;
            rect.y = root.y + (rect.y - root.y) * this.m_scale;
            rect.width *= this.m_scale;
            rect.height *= this.m_scale;
            return rect;
        }

        getBoneWroldPosition(bonename) {
            let pspBone = this.m_SpineNode.getBoneWorldPos_Root(bonename);
            if (pspBone) {
                let buff = cc.p(pspBone.x * this.getScale() * this.m_SpineNode.getScaleX(), pspBone.y * this.getScale());
                let buff1 = this.getPosition();
                return cc.pAdd(buff, buff1);
            }
            else {
                // uiMgr.getMe().show_ui_MessageBox(true, "error SkeletonAnimation_Ex::getBoneWorldPos   " + this.m_name + "没有这个骨骼 ：" + bonename);
                return null;
            }
        }
    }
}